/*
 * Provide simple functions to read the internal ADC
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib ADC Library
 *
 * @brief Provide simple functions to read the internal ADC
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */


#ifndef _ADC_H
	#include "adc.h"
	#include "default.h"
	#include <avr/io.h>
	#include <avr/interrupt.h>

	/*
	 * Dummy callback
	 */
	void adcVoid(unsigned int temp)
	{
		
	}

	void (*adc_callback)(unsigned int) = adcVoid;

	/** 
	 * @brief 	Initializes / Read an AD Channel
	 * @param 	port		AD channel to read (0 to 7)
	 * @param 	callback 	Callback function, called if conversion is finished (set to 0 in IMMEDIATE mode)
	 * @param 	type 		Conversion type (CONTINUOUS - Multiple conversions, callback if conversion is finished, SINGLE - Single conversion, callback if conversion is finished, IMMEDIATE - Single conversion, value is returned immediately)	
	 * @return 	Zero or the conversion result
	 */
	unsigned int adcInit(unsigned char port, void (*callback)(unsigned int), adcConversionType type)
	{
		unsigned char continuous = 0, interrupt = 0;

		if(type != IMMEDIATE)
		{
			adc_callback = callback;
			interrupt = 1;

		}
		if(type == CONTINUOUS)
		{
			continuous = 1;
		}

		ADMUX =  port;

		ADMUX  |= (0 << REFS0);

		ADCSRA = (1<<ADEN) | (1 << ADSC) | (continuous << ADATE) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0) | (interrupt << ADIE) ; 

		if(type == IMMEDIATE)
		{
			/* Dummy readout, prevents problems */ 
			while(ADCSRA & (1<<ADSC));

			ADCSRA |= (1<<ADSC);
			while(ADCSRA & (1<<ADSC));
		
			return ADC;
		}
		else
		{
			return 0;
		}
	}

	/*
	 * Interrupt if conversion finished
	 */
	SIGNAL (SIG_ADC)
	{
		(*adc_callback)(ADC);
	}

#endif
