/*
 * Main include file, simplify a few functions 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib Main include
 *
 * @brief Main include file, simplify a few functions
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _DEFAULT_H
	#define _DEFAULT_H   1

	/*
	 * Makros used for byte notation
	 * Example:
	 * 		i = B8(0011001100);
	 */
	#define HEX__(n) 0x##n##LU

	#define B8__(x) ((x&0x0000000FLU)?1:0) \
	+((x&0x000000F0LU)?2:0) \
	+((x&0x00000F00LU)?4:0) \
	+((x&0x0000F000LU)?8:0) \
	+((x&0x000F0000LU)?16:0) \
	+((x&0x00F00000LU)?32:0) \
	+((x&0x0F000000LU)?64:0) \
	+((x&0xF0000000LU)?128:0)

	#define B8(d) ((unsigned char)B8__(HEX__(d)))

	/*
	 * Define the bool datatype
	 */
	typedef enum
	{
		B_FALSE = 0,
		B_TRUE = 1
	} bool;


	typedef struct {
		unsigned b0:1;
		unsigned b1:1;
		unsigned b2:1; 
		unsigned b3:1;
		unsigned b4:1;
		unsigned b5:1;
		unsigned b6:1; 
		unsigned b7:1;
	} bitField;

	/*
	 * Define true & false
	 */
	#define true 1
	#define false 0

	/** 
	 * @brief 	Sleep the given number of ms
	 * @param 	mils Millisecond to sleep
	 * @return 	none
	 */
	void mSleep(int mils);

	/** 
	 * @brief 	Sleep the given number of us
	 * @param 	mils Microsecond to sleep
	 * @return 	none
	 */
	void uSleep(int mus);

	/** 
	 * @brief 	Do nothing (No operation)
	 * @param 	none
	 * @return 	none
	 */	
	void nop();

#endif
