/*
 * Read the distance using a SRF02 sonar on the i2c
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib Distance Library
 *
 * @brief Read the distance using a SRF02 sonar on the i2c
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _DISTANCE_H
	#include "distance.h"
	#include "i2c.h"

	/**
	 * @brief Start ranging
	 * @param none
	 * @return none
	 */
	void distanceStart(void)
	{
		if(!i2cStart(I2C_ADDRESS_SONAR+I2C_WRITE))
		{
			i2cWrite(0);	//Set register
			i2cWrite(81);	//Execute ranging command (in cm)
			i2cStop();		// set stop conditon = release bus
		}
	}

	/**
	 * @brief Read ranging values - start the ranging with distanceStart()
	 * @param none
	 * @return int Distance
	 */
	unsigned int distanceRead(void)
	{
		unsigned int cdistance = 0;

		if(!i2cStart(I2C_ADDRESS_SONAR+I2C_WRITE))
		{
			i2cWrite(2);		//Set register to high ranging byte
			i2cStop();
			i2cStart(I2C_ADDRESS_SONAR+I2C_READ);
			cdistance += i2cReadNak();
			i2cStop();
			cdistance <<= 8;
		
			i2cStart(I2C_ADDRESS_SONAR+I2C_WRITE);
			i2cWrite(3);		//Set register to low ranging byte
			i2cStop();
			i2cStart(I2C_ADDRESS_SONAR+I2C_READ);
			cdistance += i2cReadNak(); 
			i2cStop();
		}
	    
	    return cdistance;
	}

#endif
