/*
 * Read & Write from/to the internal EEPROM 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib EEPROM Library
 *
 * @brief Read & Write from/to the internal EEPROM 
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _EEPROM_H
	#include "eeprom.h"

	#include <stdlib.h>
	#include <avr/io.h>

	/**
	 * Write to eeprom
	 * 
	 * @param int uiAddress Eeprom address
	 * @param char ucData Data to write
	 */
	void eepromWrite(unsigned int uiAddress, unsigned char ucData)
	{
		/* Wait for completion of previous write */
		while(EECR & (1<<EEWE))
		;
		/* Set up address and data registers */
		EEAR = uiAddress;
		EEDR = ucData;
		/* Write logical one to EEMWE */
		EECR |= (1<<EEMWE);
		/* Start eeprom write by setting EEWE */
		EECR |= (1<<EEWE);
	}

	/**
	 * Read a value from a specific eeprom address
	 * 
	 * @param int uiAddress Eeprom address  
	 * @return char Eeprom value
	 */
	unsigned char eepromRead(unsigned int uiAddress)
	{
		/* Wait for completion of previous write */
		while(EECR & (1<<EEWE))
		;
		/* Set up address register */
		EEAR = uiAddress;
		/* Start eeprom read by writing EERE */
		EECR |= (1<<EERE);
		/* Return data from data register */
		return EEDR;
	}

#endif
