/*
 * Provides several functions for HD44780, KS0073 and PCF2116 (I2C) based LC displays
 * 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib LCD Library
 *
 * @brief Provides several functions for HD44780, KS0073 and PCF2116 (I2C) based LC displays
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _LCD_H
	#define _LCD_H   1

	#include "default.h"
	#include "config.h"


	#ifndef F_CPU
	#define F_CPU 16000000UL
	#endif

	#ifndef LCD_MODE
	#define LCD_MODE LCD_MODE_I2C
	#endif


	#if LCD_MODE == LCD_MODE_HD44780

	#define XTAL 					F_CPU		/**< clock frequency in Hz, used to calculate delay timer */
	#define LCD_CONTROLLER_KS0073 	0			/**< Use 0 for HD44780 controller, 1 for KS0073 controller */


	#elif LCD_MODE == LCD_MODE_KS0073

	#define XTAL 					F_CPU		/**< clock frequency in Hz, used to calculate delay timer */
	#define LCD_CONTROLLER_KS0073 	1			/**< Use 0 for HD44780 controller, 1 for KS0073 controller */



	#else //LCD_MODE == I2C

	/* I2C */

	#endif

	/**
	 * Initialize LCD
	 */
	void lcdInit(void);

	/**
	 * Clear LCD
	 */
	void lcdClear(void);

	/**
	 * Set cursor position
	 *
	 * @param char line Line no (starting with 1)
	 * @param char element Charakter index 
	 */
	void lcdPosition(unsigned char line,unsigned char element);

	/**
	 * Write string to display
	 *
	 * @param char *data String to write
	 */
	void lcdWrite(char *data);

	/**
	 * Write a int value to the display
	 *
	 * @param int value Value to write
	 */
	void lcdWriteInt(int value);

	/*
	 * Write a single sign - binary codes -> see docu
	 * 
	 * @param char data Sign to write
	 */ 
	void lcdWriteDirect(unsigned char data);

#endif
