/*
 * Provides several functions for Timer 1 based PWM generation
 * 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib PWM Library
 *
 * Provides several functions for Timer 1 based PWM generation
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _PWM_H
	#define _PWM_H   1

	/**
	 * Init and start phase frequency correct PWM output 
	 *
	 * @param uchar port Enabled PWM Ports (1 for OC1A, 2 for OC1B, 3 for both)
	 * @param uchar bits This value is used set the top counter value (8 bit TOP = 0x00FF, 9 bit TOP = 0x01FF, 10 bit TOP = 0x03FF)
	 * @param uint prescaler Clock prescaler (1, 8, 64, 256, 1024)
	 * @param uchar percentage Initial PWM output percentage (0 - 100%), for all enabled ports
	 */
	void pwmInit(unsigned char port, unsigned char bits, unsigned int prescaler, unsigned int percentage);

	/**
	 * Start PWM for all enabled pins, use this to reenable the PWM after stopping it with pwmStop()
	 */
	void pwmStart();

	/**
	 * Stop PWM for all enabled pins
	 */
	void pwmStop();

	/**
	 * Set the PWM output percentage
	 *  
	 * @param uchar port Ports (1 for OC1A, 2 for OC1B, 3 for both)
	 * @param uchar percentage Output percentage (0 - 100%)
	 */
	void pwmPercentage(unsigned char port, unsigned char percentage);

	/**
	 * Set the clock prescaler 
	 * The lower you set this value the more switching operation are performed
	 * 
	 * @param unit prescaler New prescaler (1, 8, 64, 256, 1024)
	 */
	void pwmPrescaler(unsigned int prescaler);

#endif
