/*******************************************************************
	Module: ringbuffer

*******************************************************************/
#ifndef _RINGBUFFER_H
	#define _RINGBUFFER_H

	typedef unsigned char TRingbuffer;
	typedef enum
	{
		RINGBUFFER_EMPTY,
		RINGBUFFER_SUCCESS,
		RINGBUFFER_OVERFLOW

	} TRingbufferState;

	/*******************************************************************
		Function: ringbufferInit
		Description:
			Initialise a number of ringbuffers. This defines the maximum
			simultanious used ringbuffers.
		Return:
			C51_TRUE if the wanted number of ringbuffer can be
			initialised, otherwise C51_FALSE.
	*******************************************************************/
	unsigned char
	ringbufferInit( unsigned int aNoOfringbuffers );


	/*******************************************************************
		Function: ringbufferDone
		Description:
			Free the memory. The created ringbuffers must not be used any more!!
		Return:
			C51_TRUE if the wanted number of ringbuffer can be
			initialised, otherwise C51_FALSE.

	*******************************************************************/
	unsigned char
	ringbufferDone( void );

	/*******************************************************************
		Function: ringbufferCreate
		Description:
			Create a new ringbuffer
		Return:
			TRingbuffer Created ringbuffer
	*******************************************************************/
	TRingbuffer
	ringbufferCreate( void );


	/*******************************************************************
		Function: ringbufferDestroy
		Description:
			Destroy a single ringbuffer
	*******************************************************************/
	unsigned char ringbufferDestroy(TRingbuffer aringbuffer );

	/*******************************************************************
		Function: ringbufferWrite
		Description:
			Write a databyte into the ringbuffer
		Return:
			C51_TRUE, when write was successfull
	*******************************************************************/
	unsigned char
	ringbufferWrite(
					TRingbuffer aringbuffer,
					unsigned char aDataByte);

	/*******************************************************************
		Function: ringbufferRead
		Description:
			Read a byte out of the ringbuffer
		Return:
			RINGBUFFER_EMPTY, ringbuffer is empty
			RINGBUFFER_SUCCESS, Read successfull
			RINGBUFFER_OVERFLOW, ringbuffer overflow detected
	*******************************************************************/
	TRingbufferState
	ringbufferRead(
					TRingbuffer aringbuffer,
					unsigned char *aDataByte);


#endif
