/*
 * Provide several functions for the serial(RS232) interface
 * 
 * Version: 1.0
 * Processor: ATmega641/1281/2561 (ATmega640/1280/2560 - two usarts only)
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib RS232 Library
 *
 * @brief Provide several functions for the serial(RS232) interface
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */
#ifndef _RS232_H
	#define _RS232_H 1


	#ifndef F_CPU
		#define F_CPU 12000000UL
	#endif

	/**
	 * Initialize the RS232
	 * @param	port		USART to initialize
	 * @param	baudrate	Baudrate of USART
	 * @param	rs_callback	Function called if a byte was received
	 */ 
	void rs232Init(unsigned char port, unsigned int baudrate, void (*rx_callback)(char));

	/**
	 * Write data over the serial interface 
	 * @param	xusart	Usart to write to
	 * @param	data	String to write
	 */
	void rs232Write0(unsigned char *data);
	void rs232Write1(unsigned char *data);
	#define rs232Write(XUSART, data)	rs232Write##XUSART(data)

	/**
	 * Write an integer over the serial interface 
	 * @param	xusart	Usart to write to
	 * @param	value	Integer to write
	 */
	void rs232WriteInt0(int value);
	void rs232WriteInt1(int value);
	#define rs232WriteInt(XUSART, data)	rs232WriteInt##XUSART(data)

	/**
	 * Read data from the serial interface 
	 * @param	xusart	Usart to read from
	 * @param	data	Intput string
	 * @param	length	No. of bytes to read
	 * @param	timeout	No. of tries to read a data byte, if timeout = 0 it's waited until all bytes were read, if there was a timeout the first returned byte is 'T'
	 */
	void rs232Read0(unsigned char *data, unsigned char length,unsigned int timeout);
	void rs232Read1(unsigned char *data, unsigned char length,unsigned int timeout);
	#define rs232Read(XUSART,data,length,timeout)	rs232Read##XUSART(data, length, timeout)

	/**
	 * Rs232 putchar - lowlevel functions write a single char
	 * @param	xusart	usart to write to
	 * @param	x		byte to write
	 * @return	written value
	 */
	char rs232Putchar0(char x);
	char rs232Putchar1(char x);	
	#define rs232Putchar(XUSART,data)	rs232Putchar##XUSART(data)

	/**
	 * Rs232 getchar - lowlevel functions read a single char
	 * @param	xusart	usart to read from
	 * @return	read byte
	 */	
	char rs232Getchar0();
	char rs232Getchar1();
	#define rs232Getchar(XUSART)	rs232Getchar##XUSART()	

#endif
