/*
 * Provide several SPI functions
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib SPI Library
 *
 * @brief Provide several SPI functions
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */
#ifndef _SPI_H

	#include "default.h"
	#include "spi.h"
	#include <avr/io.h>

	/**
	 * @brief Initialize SPI Master connection (current chip is master)
	 */
	void spiMasterInit(void)
	{
		/* Set MOSI and SCK output, all others input */
		DDRB |= (1<<PB0)|(1<<PB1)|(1<<PB2);
		/* Enable SPI, Master, set clock rate fck/128 */
		SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0)|(1<<SPR1)|(1<<CPHA)|(1<<CPOL);
	}

	/**
	 * @brief Write byte to slave
	 * @param data Data to transmit
	 */
	char spiMasterWrite(char cData)
	{
		//SPCR  |= (1<<MSTR); //Set SPI Master again, I don't know why but pfleurys lib always resets the bit and after 3h of debugging i just fixed it like that.
		/* Start transmission */
		SPDR = cData;
		/* Wait for transmission complete */
		while(!(SPSR & (1<<SPIF)));
		
		return SPDR;
	}
	/**
	 * @brief Initialize SPI slave connection (current chip is slave)
	 */
	void spiSlaveInit(void)
	{
		/* Set MISO output, all others input */
		DDRB |= (1<<PB6);
		/* Enable SPI */
		SPCR = (1<<SPE);
	}

	/**
	 * @brief Read byte from master
	 * @return Read byte
	 */	
	unsigned char spiSlaveRead(void)
	{
		/* Wait for reception complete */
		while(!(SPSR & (1<<SPIF)))
		;
		/* Return data register */
		return SPDR;
	}

#endif
