#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include "config.h"
#include "default.h"
#include "suart.h"

#ifdef SUART_TXD
    static volatile uint16_t outframe;
#endif

#ifdef SUART_RXD
    static volatile uint16_t inframe;
    static volatile uint8_t inbits, received;

    static volatile uint8_t indata;
#endif
 
void rs232Init10(unsigned int baudrate)
{
    uint8_t tifr = 0;
    uint8_t sreg = SREG;
    cli();

    // Mode #4 fr TimeR3 
    // und volle MCU clock 
    // IC Noise Cancel 
    // IC on Falling Edge 
    TCCR3A = 0;
    TCCR3B = (1 << WGM32) | (1 << CS30) | (0 << ICES3) | (1 << ICNC3);

    // OutputCompare fr gewnschte TimeR3 Frequenz 
    OCR3A = (uint16_t) ((uint32_t) F_CPU/baudrate);

#ifdef SUART_RXD
    SUART_RXD_DDR  &= ~(1 << SUART_RXD_BIT);
    SUART_RXD_PORT |=  (1 << SUART_RXD_BIT);
    TIMSK3 |= (1 << ICIE3);
    TIFR3  |= (1 << ICF3) | (1 << OCF3B);
#else
    TIMSK3 &= ~(1 << ICIE3);
#endif // SUART_RXD 

#ifdef SUART_TXD
    TIFR3 |= (1 << OCF3A);
    SUART_TXD_PORT |= (1 << SUART_TXD_BIT);
    SUART_TXD_DDR  |= (1 << SUART_TXD_BIT);
    outframe = 0;
#endif // SUART_TXD 

    TIFR3 = tifr;

    SREG = sreg;
}

#ifdef SUART_TXD
void rs232Putchar10(const char c)
{
    do
    {
        sei(); nop(); cli(); // yield(); 
    } while (outframe);

    // frame = *.P.7.6.5.4.3.2.1.0.S   S=Start(0), P=Stop(1), *=Endemarke(1) 
    outframe = (3 << 9) | (((uint8_t) c) << 1);

    TIMSK3 |= (1 << OCIE3A);
    TIFR3   = (1 << OCF3A);

    sei();
}
void rs232Write10(unsigned char *data)
{	
	unsigned char c;
	while ( (c = *data++) ) 
	{
		rs232Putchar10(c);
	}
}

#endif // SUART_TXD 

#ifdef SUART_TXD
SIGNAL (SIG_OUTPUT_COMPARE3A)
{
    uint16_t data = outframe;
   
    if (data & 1)      SUART_TXD_PORT |=  (1 << SUART_TXD_BIT);
    else               SUART_TXD_PORT &= ~(1 << SUART_TXD_BIT);
   
    if (1 == data)
    {
        TIMSK3 &= ~(1 << OCIE3A);
    }   
   
    outframe = data >> 1;
}
#endif // SUART_TXD

#ifdef SUART_RXD
SIGNAL (SIG_INPUT_CAPTURE3)
{
    uint16_t icr3  = ICR3;
    uint16_t ocr3a = OCR3A;
   
    // Eine halbe Bitzeit zu ICR3 addieren (modulo OCR3A) und nach OCR3B
    uint16_t ocr3b = icr3 + ocr3a/2;
    if (ocr3b >= ocr3a)
        ocr3b -= ocr3a;
    OCR3B = ocr3b;
   
    TIFR3 = (1 << OCF3B);
    TIMSK3 = (TIMSK3 & ~(1 << ICIE3)) | (1 << OCIE3B);
    inframe = 0;
    inbits = 0;
}
#endif // SUART_RXD

#ifdef SUART_RXD
SIGNAL (SIG_OUTPUT_COMPARE3B)
{
    uint16_t data = inframe >> 1;
   
    if (SUART_RXD_PIN & (1 << SUART_RXD_BIT))
        data |= (1 << 9);
      
    uint8_t bits = inbits+1;
   
    if (10 == bits)
    {
        if ((data & 1) == 0)
            if (data >= (1 << 9))
            {
                indata = data >> 1;
                received = 1;
            }
      
        TIMSK3 = (TIMSK3 & ~(1 << OCIE3B)) | (1 << ICIE3);
        TIFR3 = (1 << ICF3);
    }
    else
    {
        inbits = bits;
        inframe = data;
    }
}
#endif // SUART_RXD

#ifdef SUART_RXD

char rs232Getchar10()
{
    while (!received){}
    received = 0;
   
    return indata;
}

int rs232GetcharNw10()
{
    if (received)
    {
        received = 0;
        return (int) indata;
    }
   
    return -1;
}
void rs232Read10(unsigned char *data, unsigned char length,unsigned int timeout)
{
		unsigned char i = 0;
		unsigned int  time = 0;
		int temp = 0;

		/* non blocking */
		if (timeout != 0) {
			while (i < length && time++ < timeout) {
				if ( (temp = rs232GetcharNw10()) != -1) {
					data[i++] = (char)temp;
					time = 0;
				}
			}
			if (time > timeout) data[0] = 0;
		}
		/* blocking */
		else {
			while (i < length) {
				data[i++] = rs232Getchar10();
			}
		}
}
#endif // SUART_RXD
