#include "board.h"
#include "blib/timer.h"
#include <avr/io.h>
#include <avr/interrupt.h>

unsigned char board_led_hearbeat = 0;
unsigned int board_led_heartbeat_count = 0;
unsigned int board_btimer_count = 0;

void boardInit()
{
	DDRC |= (1 << PC3) | (1 << PC2); //Set LEDs as output
}

void boardLedWrite(unsigned char id, unsigned char value)
{
	if(id == BOARD_LED_GREEN)
	{
		if(value)
		{
			PORTC |= (1 << PC3);
		}
		else
		{
			PORTC &= ~(1 << PC3);
		}
	}
	else if(id == BOARD_LED_RED)
	{
		if(value)
		{
			PORTC |= (1 << PC2);
		}
		else
		{
			PORTC &= ~(1 << PC2);
		}
	}
}

void boardLedHeartbeat()
{
	if(board_led_hearbeat == 0 && board_led_heartbeat_count >= 300)
	{
		boardLedWrite(BOARD_LED_RED, 1);
		board_led_hearbeat++;
		board_led_heartbeat_count = 0;
	}
	else if(board_led_hearbeat == 1 && board_led_heartbeat_count >= 60)
	{
		boardLedWrite(BOARD_LED_RED, 0);
		board_led_hearbeat++;
		board_led_heartbeat_count = 0;		
	}
	else if(board_led_hearbeat == 2 && board_led_heartbeat_count >= 30)
	{
		boardLedWrite(BOARD_LED_RED, 1);
		board_led_hearbeat++;
		board_led_heartbeat_count = 0;		
	}
	else if(board_led_hearbeat == 3 && board_led_heartbeat_count >= 60)
	{
		boardLedWrite(BOARD_LED_RED, 0);
		board_led_hearbeat = 0;
		board_led_heartbeat_count = 0;		
	}
	else
	{
		board_led_heartbeat_count++;
	}
	board_btimer_count++;
}

void boardLedHeartbeatInit()
{
	timerInit(1, 100, boardLedHeartbeat);
}

unsigned int boardGetBTimer()
{
	unsigned int tmp;
	cli(); /* Atomic operation - 16 bit value -> 2 clock cycles */
	tmp = board_btimer_count;
	sei();
	return tmp;
}

void boardResetBTimer()
{
	cli(); /* Atomic operation - 16 bit value -> 2 clock cycles */
	board_btimer_count = 0;
	sei();	
}
