
#ifndef _SDFAT_H

	#define _SDFAT_H

	#include "dos.h"
	#include <avr/io.h>

	/*
	 * Initialize SDFat
	 */
	void sdfatInit();

	/*
	 * Load FAT Filesystem
	 * This function waits until a valid system has been loaded
	 * maximal 100 retires
	 */
	void sdfatLoad();

	/*
	 * Get the capacity of the inserted SD Card
	 * @return Capacity of the SD Card in MB
	 */
	unsigned int sdfatCapacity();	

	/*
	 * Open a file and return a file pointer
	 * @param name Filename in the filesystem
	 * @param flag Dos known flags / F_READ, F_WRITE or F_APPEND
	 * @return file pointer
	 */
	unsigned char sdfatOpen(char *name, unsigned char flag);

	/*
	 * Seek in the file
	 * @param offset Offset to seek too
	 * @param mode Available mode \
	 *	SEEK_SET Seek from beginning of file. offset has to be positive. \
	 *  SEEK_CUR Seek from current file pointer position. offset may be positive or negative. \
	 *  SEEK_END Seek from end of file. offset has to be negative.
	 */
	unsigned char sdfatSeek(int offset, unsigned char mode);
	
	/*
	 * Read from a file
	 * @param buf Buffer where your data should be stored
	 * @param count Number of bytes to be read
	 * @return Number of bytes read from the file
	 */	
	unsigned int sdfatRead(unsigned char *buf, unsigned int count);


	/*
	 * Write to a file
	 * @param buf Buffer containing your data
	 * @param count Number of bytes to be written
	 * @return Number of bytes written to the file
	 */
	unsigned int sdfatWrite(unsigned char *buf, unsigned int count);

	/*
	 * Close a opened file
	 */
	void sdfatClose();

#endif 
	
	
