#ifndef _BMP_H

	#include <avr/interrupt.h>
	#include "glcd.h"
	#include "bmp.h"
	#include "../fat/sdfat.h"
	#include "../blib/default.h"


	void bmpInit()
	{
		sdfatInit();		
		sdfatLoad();		
	}
	
	unsigned char bmpLoad(char *file, unsigned char page, unsigned char column)
	{
		unsigned char bmpBuffer[BMP_displayWidth];
		unsigned char bmpLinebuffer[BMP_blockSize];	
	
		unsigned char readFile, pages;
		unsigned char i = 0, k = 0, l = 0, m = 0, c = 0;

		unsigned int bmpOffset = 0;
		unsigned int bmpWidth = 0;
		unsigned int bmpHeight = 0;
		unsigned int bmpBlockSize = 0;

		cli();
		readFile = sdfatOpen(file, 'r');

		if (readFile==F_OK)
		{
			sdfatSeek(BMP_bfOffBits, SEEK_SET);
			bmpOffset = bmpReadInt();

			sdfatSeek(BMP_biWidth, SEEK_SET);
			bmpWidth = bmpReadInt();

			sdfatSeek(BMP_biHeight, SEEK_SET);
			bmpHeight = bmpReadInt();

			bmpBlockSize = 4 * ( ( bmpWidth + 31 ) / 32 );

			sdfatSeek(bmpOffset, SEEK_SET);		

			pages = bmpHeight / 8;

			for(m = 0; m < pages; m++)
			{
				for(k = 0; k < bmpWidth; k++)
				{
					bmpBuffer[k] = 0;
				}

				//Read 8 lines and shift them into the buffer
				for(k = 0; k < 8; k++)
				{
					sdfatRead(bmpLinebuffer, bmpBlockSize);
				
					for(i = 0; i < bmpBlockSize; i++)
					{
						c = i*8;

						for(l = 0; l < 8; l++)
						{
							bmpBuffer[c+l] |= BMP_EXTRACT_BIT(~bmpLinebuffer[i], 7-l) << (7 - k);
						}
					}
				}
				glcdWriteDataLine(bmpBuffer, bmpWidth, page + (pages - m - 1), column);
			}

			sdfatClose();
		
			sei();

			return 1;
		}
		else
		{
			printf("Loading of file %s failed\n", file);
		
			sei();

			return 0;
		}
	}

	unsigned int bmpReadInt()
	{
		unsigned char i = 0, tmp[2];
		unsigned int value = 0;
				
		while(i < 4 && sdfatRead(tmp, 1))
		{
			value |= tmp[0] << i;
			i++;
		}
		return value;	
	}

#endif
