#include "blib/config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include "board.h"
#include "blib/adc.h"
#include "blib/spi.h"
#include "blib/timer.h"
#include "blib/default.h"
#include "glcd/glcd.h"
#include "glcd/multiline.h"
#include "glcd/bmp.h"
#include "blib/rs232.h"
#include "ui/Ui.h"
#include "ui/Accelerometer.h"
#include "dialogs/DialogMain.h"

/* 
 * Timer Usage:
 * T1 --> Heartbeat (board)
 * T3 --> Serial USART
 *
 */


//#define XREMOTE_LONG_BOOT
#define XREMOTE_SILENT_BOOT
 
void splashScreen();


int	main (void) 
{
	char data[32];

	static FILE rs232_stream = FDEV_SETUP_STREAM(rs232Putchar0,	rs232Getchar0, _FDEV_SETUP_RW);
	stdout = &rs232_stream;


	glcdInit();
	
	//Enable Backlight
	DDRC |= B8(01000000);
	/*
	PORTC |= B8(01000000);
	*/

	boardInit();
	boardLedHeartbeatInit();

#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("      xOS 3.33      ");
	multilineAdd(" Copyright 2009 BWA ");
	multilineAdd("--------------------");
	multilineAdd("CPU:      ATmega1281");
	sprintf(data,"Speed:        %2u MHz", F_CPU/1000000UL);
	multilineAdd(data);
	sprintf(data,"Baudrate:     %6u", BAUDRATE);
	multilineAdd(data);
	sprintf(data,"I2C:         %3u kHz", SCL_CLOCK/1000UL);
	multilineAdd(data);
	multilineAdd("--------------------");
#ifdef XREMOTE_LONG_BOOT
	mSleep(1000);
	mSleep(1000);
#endif
	multilineAdd("Screen:     128x64px");
	multilineAdd("Cap. Sensor:  QT1106");
	multilineAdd("Accelerom.: LIS3LV02");
	multilineAdd("USB.:         CP2102");
	multilineAdd("Battery:  1.1AH LiPO");
	multilineAdd("--------------------");
	multilineAdd("Booting...");

	#ifdef XREMOTE_LONG_BOOT
		mSleep(1000);
		mSleep(1000);
	#endif
	multilineAdd("Initializing BT...");
	multilineAdd("BT initialized!");
	#ifdef XREMOTE_LONG_BOOT
		mSleep(1000);
	#endif
#endif

#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("Initializing SD...");
#endif
	bmpInit();
#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("SD Card initialized!");
	sprintf(data,"microSD:     %4u MB", sdfatCapacity());
	multilineAdd(data);
	#ifdef XREMOTE_LONG_BOOT
		mSleep(1000);
	#endif
#endif

#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("Initializing UI...");
#endif
	uiInit();
#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("UI initialized!");

	#ifdef XREMOTE_LONG_BOOT
		mSleep(1000);
	#endif
#endif

#ifndef XREMOTE_SILENT_BOOT
	multilineAdd("System ready!");
	multilineAdd("Starting Application");
	#ifdef XREMOTE_LONG_BOOT
		mSleep(1000);
		mSleep(1000);
	#endif
#endif

	splashScreen();


	rs232Init(10, 38400, 0);
	sei();
	rs232Write(10, "\r\nWelcome!");

	dialogMain();


	for (;;) 
	{
		uiMain();
	}
}


void splashScreen()
{
	cli();
	bmpLoad("splash.bmp", 0, 0);
	mSleep(1000);
#ifdef XREMOTE_LONG_BOOT
	mSleep(1000);
#endif
	sei();
}
