// Module..: WII_IR_SENSOR.C
// Version.: 1.0
// Compiler: CodeVisionAVR
// Chip....: ATmega88
// Date....: July 2008
// Author..: Udo Juerss
//-----------------------------------------------------------------------------


#include "irsensor.h"
#include "avr/io.h"
//-----------------------------------------------------------------------------

unsigned char IRS_SENSITIVITY_SETTINGS[5][4] = 
{
	{0x72, 0x20, 0x1F, 0x03},
	{0xC8, 0x36, 0x35, 0x03},
	{0xAA, 0x64, 0x63, 0x03},
	{0x96, 0xB4, 0xB3, 0x04},
	{0x96, 0xFE, 0xFE, 0x05}
};


unsigned char irs_data[IRS_I2C_BUFFER_SIZE];
tWII_POINT irs_points[IRS_MAX_POINTS];
tWII_IIR_POINT irs_iir_points[IRS_MAX_POINTS];

unsigned char irs_present;
//-----------------------------------------------------------------------------

void irsWriteDoubleByte(unsigned char byte1,unsigned char byte2)
{
  if(!i2cStart(IRS_SALVE_ADDR & ~I2C_WRITE))
  {
  	i2cWrite(byte1);
	i2cWrite(byte2);

	i2cStop();

	irs_present = 1;
  }
  else
  {
  	irs_present = 0;
  }
}
//-----------------------------------------------------------------------------

void irsSensitivity(unsigned char sensitivity)
{
  if(!i2cStart(IRS_SALVE_ADDR & ~I2C_WRITE))
  {
  	i2cWrite(0x00);
	i2cWrite(0x02);
	i2cWrite(0x00);
	i2cWrite(0x00);
	i2cWrite(0x71);
	i2cWrite(0x01);
	i2cWrite(0x00);
	i2cWrite(IRS_SENSITIVITY_SETTINGS[sensitivity][0]);
	i2cStop();

	irs_present = 1;
  }
  else
  {
  	irs_present = 0;
  }
  if(!i2cStart(IRS_SALVE_ADDR & ~I2C_WRITE))
  {
  	i2cWrite(0x07);
	i2cWrite(0x00);
	i2cWrite(IRS_SENSITIVITY_SETTINGS[sensitivity][1]);
	i2cStop();

	irs_present = 1;
  }
  else
  {
  	irs_present = 0;
  }
   
  if(!i2cStart(IRS_SALVE_ADDR & ~I2C_WRITE))
  {
  	i2cWrite(0x1A);
	i2cWrite(IRS_SENSITIVITY_SETTINGS[sensitivity][2]);
	i2cWrite(IRS_SENSITIVITY_SETTINGS[sensitivity][3]);
	i2cStop();

	irs_present = 1;
  }
  else
  {
  	irs_present = 0;
  }	
}

// Initialize the Wii IR Sensor
void irsInit(void)
{
  i2cInit();
  
  DDRD &= ~(1 << PD6);

  if(PIND & (1 << PD6)) // Reset Line pulled high
  {
  	  /* RESET PULSE */
 	  DDRD |= (1 << PD6);
	  PORTD &= ~(1 << PD6);
	  mSleep(1);
 	  PORTD |= (1 << PD6);	
      DDRD &= ~(1 << PD6);   	
	  mSleep(10);

	  irs_present = true;

	  // Send initialization data to sensor
	  irsWriteDoubleByte(0x30,0x01);

	  if(!irs_present){
		/* Disable LED Panel */	  	
	    PORTD &= ~(1 << PD6);
	    PORTB &= ~((1 << PB6) | (1 << PB7));		
		return;
	  }

	  /*
	  Enable LED PANEL 
	  */
	  DDRD |= (1 << PD7);
	  DDRB |= (1 << PB6) | (1 << PB7);
	  PORTD |= (1 << PD7);
	  PORTB |= (1 << PB6) | (1 << PB7);

	  irsSensitivity(2);
	  irsWriteDoubleByte(0x33,0x03);
	  irsWriteDoubleByte(0x30,0x08);

	  mSleep(100);

  }

}
//-----------------------------------------------------------------------------

// Read raw data from I2C bus
void irsRead(void)
{
  unsigned char n;

  if (irs_present)
  {
    // Prepare next read cycle
	i2cStart(IRS_SALVE_ADDR & ~I2C_WRITE);

	i2cWrite(0x36);

    mSleep(IRS_ACCESS_DELAY_MS);

	i2cStop();

	i2cRepStart(IRS_SALVE_ADDR | I2C_READ);

    // Read next position data
    for (n = 0; n < IRS_I2C_BUFFER_SIZE; n++)
    {
		if(n == IRS_I2C_BUFFER_SIZE - 1)
		{
			irs_data[n] = i2cReadNak();
		}
		else
		{									
			irs_data[n] = i2cReadAck();
		}
	  
    }

	i2cStop();

    mSleep(IRS_ACCESS_DELAY_MS);
  }
}
//-----------------------------------------------------------------------------

void irsConvert(void)
{
  unsigned int temp;

  // Convert data bytes to points
  irs_points[0].xpos = irs_data[1];
  irs_points[0].ypos = irs_data[2];
  temp = irs_data[3];
  irs_points[0].xpos += (temp & 0x30) << 4;
  irs_points[0].ypos += (temp & 0xC0) << 2;

  irs_points[1].xpos = irs_data[4];
  irs_points[1].ypos = irs_data[5];
  temp = irs_data[6];
  irs_points[1].xpos += (temp & 0x30) << 4;
  irs_points[1].ypos += (temp & 0xC0) << 2;

  irs_points[2].xpos = irs_data[7];
  irs_points[2].ypos = irs_data[8];
  temp = irs_data[9];
  irs_points[2].xpos += (temp & 0x30) << 4;
  irs_points[2].ypos += (temp & 0xC0) << 2;

  irs_points[3].xpos = irs_data[10];
  irs_points[3].ypos = irs_data[11];
  temp = irs_data[12];
  irs_points[3].xpos += (temp & 0x30) << 4;
  irs_points[3].ypos += (temp & 0xC0) << 2;


  // Calculate IIR filter values for all points
  irs_iir_points[0].xpos = irs_iir_points[0].xpos * IRS_IIR_COEFF +
    (float)irs_points[0].xpos * IRS_IIR_DIFF_COEFF;
  irs_iir_points[0].ypos = irs_iir_points[0].ypos * IRS_IIR_COEFF +
    (float)irs_points[0].ypos * IRS_IIR_DIFF_COEFF;

  irs_iir_points[1].xpos = irs_iir_points[1].xpos * IRS_IIR_COEFF +
    (float)irs_points[1].xpos * IRS_IIR_DIFF_COEFF;
  irs_iir_points[1].ypos = irs_iir_points[1].ypos * IRS_IIR_COEFF +
    (float)irs_points[1].ypos * IRS_IIR_DIFF_COEFF;

  irs_iir_points[2].xpos = irs_iir_points[2].xpos * IRS_IIR_COEFF +
    (float)irs_points[2].xpos * IRS_IIR_DIFF_COEFF;
  irs_iir_points[2].ypos = irs_iir_points[2].ypos * IRS_IIR_COEFF +
    (float)irs_points[2].ypos * IRS_IIR_DIFF_COEFF;

  irs_iir_points[3].xpos = irs_iir_points[3].xpos * IRS_IIR_COEFF +
    (float)irs_points[3].xpos * IRS_IIR_DIFF_COEFF;
  irs_iir_points[3].ypos = irs_iir_points[3].ypos * IRS_IIR_COEFF +
    (float)irs_points[3].ypos * IRS_IIR_DIFF_COEFF;

}
//-----------------------------------------------------------------------------

void irsUpdate(void)
{
	irsRead();
	irsConvert();
}

unsigned char irsPresent(unsigned char init)
{
	if(!irs_present && init)
	{
		irsInit();
	}
	return irs_present;
}
