/*============================================================================
    Project     QT1106 Demo code
    File        qt1106.h
    Function    Header file for QT1106 driver (qt1106.c) 
    Requires    
    Originated	D Spokes
    
    (c) Quantum Research Group
============================================================================*/
 
/* Multiple inclusion protection for headers */

#ifndef _QT1106_H
#define _QT1106_H

enum {	/* QT Command fields used when calling SetQtCommandField()
           and GetQtCommandField() */
	SET_QT_PROX,
	SET_QT_SLD,
	SET_QT_AKS,
	SET_QT_MOD,
	SET_QT_DI,
	SET_QT_LPB,
	SET_QT_MODE,
	SET_QT_RESOLUTION
	};

/* the following enums list command values used when calling SetQtCommandField()
   and returned by GetQtCommandField() - refer QT1106 datasheet */

enum { /* command values for the PROX bit */
	QT_PROX_OFF,
	QT_PROX_ON = 0x10
	};

enum { /* command values for the SLD bit */
	QT_MODE_WHEEL,
	QT_MODE_SLIDER = 0x08
	};

enum { /* command values for the AKS bits */
	QT_AKS_DISABLED,
	QT_AKS_GLOBAL,
	QT_AKS_K_PLUS_W_S,
	QT_AKS_4K_PLUS_3K_PLUS_W_S,
	QT_AKS_4K_PLUS_3K_AND_W_S,
	QT_AKS_6K_AND_W_S_PLUS_K7
	};

enum { /* command values for the MOD bits */
	QT_MOD_10S,
	QT_MOD_20S = 0x20,
	QT_MOD_60S = 0x40,
	QT_MOD_INFINITE = 0x60
	};

enum { /* command values for the DI bit */
	QT_DI_2_DETECTS,
	QT_DI_6_DETECTS = 0x10
	};

enum { /* command values for the LPB bit */
	QT_LPB_OFF,
	QT_LPB_ON = 0x08
	};


enum { /* command values for the MODE bits */
	QT_MODE_FREE_RUN,
	QT_MODE_LP_120,
	QT_MODE_LP_200,
	QT_MODE_LP_360,
	QT_MODE_LP_680,
	QT_MODE_SYNC,
	QT_MODE_SLEEP
	};

enum { /* command values for the RESOLUTION bits */
	QT_RESOLUTION_2 = 0x20,
	QT_RESOLUTION_3	= 0x40,
	QT_RESOLUTION_4 = 0x60,
	QT_RESOLUTION_5 = 0x80,
	QT_RESOLUTION_6 = 0xa0,
	QT_RESOLUTION_7 = 0xc0,
	QT_RESOLUTION_8 = 0xe0
	};

/* comms modes used when calling SetQt1106CommsMode() */
enum {
	QT_COMMS_MODE_OFF,
	QT_COMMS_MODE_POLL_ALWAYS,
	QT_COMMS_MODE_POLL_ASYNC,
	QT_COMMS_MODE_POLL_ON_CHANGE
	};

 /* command codes used when calling CommandQt() */
enum {
	QT_COMMAND_CAL_WHEEL,
	QT_COMMAND_CAL_KEY,
	QT_COMMAND_SET_THRESHOLD
	};

/* Fields used when calling GetQtKeyState() */
enum {
	QT_FLAG_STATE,
	QT_KEY_STATE,
	QT_WHEEL_POSITION,
	QT_THRESHOLD
	};

/* PUBLIC FUNCTIONS */

void InitQt1106 ( void );
void SetQt1106CommsMode ( unsigned char Mode, unsigned int PollRate );
unsigned char ProcessQt1106( void );
void CommandQt ( unsigned char Command, unsigned char Arg );
void SetQtCommandField ( unsigned char Field, unsigned char Value );
unsigned char GetQtCommandField ( unsigned char Field );
unsigned char GetQtStatus ( unsigned char Field );



#endif /* _QT1106_H */
