#ifndef _TIF_H
	#define _TIF_H

	#define TIF_MAX_EVENTS 32

	typedef struct
	{
		unsigned char uid;
		unsigned int x;
		unsigned int y;
		unsigned int width;
		unsigned int height;
	} TifEvent;

	/* 
	 * Initialize Touch Interface
	 * @param callback Function to be called if an event occurs
	 */
	void tifInit(void (*callback)(unsigned int, unsigned int, unsigned char));
	
	/*
	 * Register a new user event (a event is a click in a certain square area)
	 * @param uid ID of the event
	 * @param x X Coordinate of the top left corner of the square
	 * @param y Y Coordinate of the top left corner of the square
	 * @param width Width of the event square
	 * @param height Height of the event square
	 */
	unsigned char tifEventRegister(unsigned char uid, unsigned int x, unsigned int y, unsigned int width, unsigned int height);
	
	/*
	 * Clear all events
	 */
	void tifEventClearAll();

	/*
	 * Remove a specific event, by it's ID
	 */
	void tifEventClear(unsigned char event);


	/* Private */
	void tifEventCallback(unsigned int x, unsigned int y, unsigned char mask);
	unsigned char tifInArea(unsigned char screen_x, unsigned char screen_y, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2);


#endif
