#ifndef _TOUCH_H

	#include "touch.h"
	#include "../blib/adc.h"
	#include <avr/io.h>

	void (*touch_position_callback)(unsigned int, unsigned int, unsigned char) = 0;
	
	unsigned char touch_count = 0;
	unsigned char touch_channel = 0;
	unsigned int  touch_data[2][TOUCH_MIDDLE_VALUES];

	static void touchSetY1(unsigned char stat)
	{
		if(!stat)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_Y1);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_Y1);			
		}
		else if(stat == 1)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_Y1);
			TOUCH_PORT_ADC |= (1 << TOUCH_PIN_Y1);
		}
		else
		{
			TOUCH_DDR_ADC &= ~(1 << TOUCH_PIN_Y1);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_Y1);
		}
	}

	static void touchSetY2(unsigned char stat)
	{
		if(!stat)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_Y2);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_Y2);			
		}
		else if(stat == 1)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_Y2);
			TOUCH_PORT_ADC |= (1 << TOUCH_PIN_Y2);
		}
		else
		{
			TOUCH_DDR_ADC &= ~(1 << TOUCH_PIN_Y2);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_Y2);
		}
	}

	static void touchSetX1(unsigned char stat)
	{
		if(!stat)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_X1);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_X1);			
		}
		else if(stat == 1)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_X1);
			TOUCH_PORT_ADC |= (1 << TOUCH_PIN_X1);
		}
		else
		{
			TOUCH_DDR_ADC &= ~(1 << TOUCH_PIN_X1);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_X1);
		}
	}

	static void touchSetX2(unsigned char stat)
	{
		if(!stat)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_X2);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_X2);			
		}
		else if(stat == 1)
		{
			TOUCH_DDR_ADC |= (1 << TOUCH_PIN_X2);
			TOUCH_PORT_ADC |= (1 << TOUCH_PIN_X2);
		}
		else
		{
			TOUCH_DDR_ADC &= ~(1 << TOUCH_PIN_X2);
			TOUCH_PORT_ADC &= ~(1 << TOUCH_PIN_X2);
		}
	}

	void touchInit(void (*position_callback)(unsigned int, unsigned int, unsigned char))
	{
		touch_position_callback = position_callback;
		touchStart();
	}

	void touchStart()
	{
		touchSetX1(2);
		touchSetX2(2);

		touchSetY1(0);
		touchSetY2(1);

		touch_channel = 0;
		touch_count = 0;

		adcInit(TOUCH_PIN_X1, touchAdcCallback, CONTINUOUS);
	}

	void touchStop()
	{
		adcInit(TOUCH_PIN_X1, 0, IMMEDIATE); //Dummy readout, just to get away from continous mode
	}

	//Should be called at least with a frequency of 2khz
	void touchAdcCallback(unsigned int value)
	{
		if(touch_count < TOUCH_MIDDLE_VALUES)
		{
			touch_data[touch_channel][touch_count++] = value;
			return;
		}

		if(touch_channel == 0)
		{
			touchSetX1(0);
			touchSetX2(1);

			touchSetY1(2);
			touchSetY2(2);

			adcInit(TOUCH_PIN_Y1, touchAdcCallback, CONTINUOUS);

			touch_channel = 1;
			touch_count = 0;
		}
		else if(touch_channel == 1)
		{
			touchSetX1(2);
			touchSetX2(2);

			touchSetY1(0);
			touchSetY2(1);

			adcInit(TOUCH_PIN_X1, touchAdcCallback, CONTINUOUS);
			
			touch_channel = 0;
			touch_count = 0;

			//Both channels read - compare with last values
			unsigned int x = touchReadX();
			unsigned int y = touchReadY();			

			if(x != touch_last_x || y != touch_last_y)
			{
				if(x == TOUCH_NOT_PRESSED || y == TOUCH_NOT_PRESSED)
				{
					touch_position_callback(TOUCH_NOT_PRESSED, TOUCH_NOT_PRESSED, 0x00);
				}
				else
				{
					touch_position_callback(x, y, 0x01);
				}

				touch_last_x = x;
				touch_last_y = y;
			}
		}			
		
		
		

	}

	unsigned int touchReadX()
	{
		unsigned long tmp = 0;
		unsigned char i = 0;

		for(i = 0; i < TOUCH_MIDDLE_VALUES; i++)
		{
			tmp += touch_data[0][i];
		}

		tmp /= TOUCH_MIDDLE_VALUES;

		if(tmp >= 10)
		{		
	 		if(tmp < TOUCH_X_ADC_START){tmp = TOUCH_X_ADC_START;}
			if(tmp > TOUCH_X_ADC_END){tmp = TOUCH_X_ADC_END;}

			tmp -= TOUCH_X_ADC_START;

			tmp = (TOUCH_SCREEN_WIDTH * tmp) / TOUCH_X_ADC_WIDTH;

			return (int)tmp;	
		}
		else
		{
			return TOUCH_NOT_PRESSED;
		}
	}

	unsigned int touchReadY()
	{
		unsigned long tmp = 0;
		unsigned char i = 0;

		for(i = 0; i < TOUCH_MIDDLE_VALUES; i++)
		{
			tmp += touch_data[1][i];
		}

		tmp /= TOUCH_MIDDLE_VALUES;

		if(tmp >= 10)
		{		
	 		if(tmp < TOUCH_Y_ADC_START){tmp = TOUCH_Y_ADC_START;}
			if(tmp > TOUCH_Y_ADC_END){tmp = TOUCH_Y_ADC_END;}

			tmp -= TOUCH_Y_ADC_START;

			tmp = (TOUCH_SCREEN_HEIGHT * tmp) / TOUCH_Y_ADC_HEIGHT;

			return (int)tmp;
		}
		else
		{
			return TOUCH_NOT_PRESSED;
		}
	}

	unsigned char touchInArea(unsigned char screen_x, unsigned char screen_y, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
	{
		if(screen_x >= x1 && screen_x <= x2 && screen_y >= y1 && screen_y <= y2)
		{
			return 1;
		}
		return 0;
	}


#endif
