/*
 * xMedia
 * User Interface Control
 *
 * STATUS Commands
 * 
 * 0x01 - OK -> CMD_OK
 * 	Payload data:
 * 		no data expected
 * 	Return value:
 * 		no return value	
 * 
 * 0x02 - Failure -> CMD_FAIL
 * 	Payload data:
 * 		error code
 * 	Return value:
 * 		no return value	
 * 
 * 0x03 - Busy - CMD_BUSY
 * 	Payload data:
 * 		error code
 * 	Return value:
 * 		no return value	
 * 
 * 
 * INFO Commands
 * 
 * 0x11 - Check status
 * 	Payload data:
 * 		no data expected
 * 	Return value expected:
 * 		CMD_OK 		if device is ready
 * 		CMD_BUSY 	if device is busy
 * 		timeout 	if device is busy
 * 
 * POWER MANAGEMENT Commands 
 * 
 * 0x21 - Enable/Disable LCD Backlight and 5V Supply
 * 	Payload data:
 * 		0x00 disable
 * 		0x01 to 0xFF enabled
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed			
 * 
 * 0x22 - Set LCD Backlight current
 * 	Payload data:
 * 		0x00 	minimal current
 * 		0xFF 	maximal current
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed
 * 
 * 0x23 - Enable/Disable VGA Output
 * 	Payload data:
 * 		0x00 disable
 * 		0x01 to 0xFF enabled
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed
 * 
 * 
 * UI commands
 * 
 * 0x31 - Button OK
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x32 - Button Back
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x33 - Button GP1
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x34 - Button GP2
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x35 - Right
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x36 - Left
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * Mouse/Touchpad emulation
 *
 * 0x51 - Absolute Position
 * 	Payload data:
 * 		data[0..1]	x coordinate - unsigned integer
 *		data[2..3]	y coordinate - unsigned integer
 *		data[4]		Left button pressed
 * 		data[5]		Right button pressed
 * 	Return value:
 * 		no return value	
 *
 * 0x52 - Relative Position
 * 	Payload data:
 * 		data[0..1]	x increment - signed integer
 *		data[2..3]	y coordinate - signed integer
 *		data[4]		Left button pressed
 * 		data[5]		Right button pressed
 * 	Return value:
 * 		no return value	
 */

#include "Ui.h"
#include "CapInterface.h"
#include "Xdi.h"
#include "Accelerometer.h"
#include "../board.h"
#include "../sie/Sie.h"
#include "../blib/default.h"
#include "../blib/rs232.h"
#include "../blib/config.h"
#include "../board.h"


unsigned char ui_sie_id = 0;
void (*ui_cmd_callback)(unsigned char) = uiSubmitCommand;
void (*ui_position_callback)(unsigned int, unsigned int, unsigned char) = uiSubmitPosition;
unsigned int ui_screen_width = 128;
unsigned int ui_screen_height = 64;
unsigned int ui_time = 0;
unsigned int ui_last_time = 0;

unsigned char ui_routing[10] = {0,0,0,0,0,0,0,0,0,0};



unsigned int uiGetRealX(unsigned int x)
{
	return (x / 1000.0)*ui_screen_width;
}

unsigned int uiGetRealY(unsigned int y)
{
	return (y / 1000.0)*ui_screen_height;
}

void uiPacketReceived(siePacket packet)
{
	
}
unsigned int time = 0;

void uiXdiCommandChange(unsigned char cmd)
{
	if(ui_last_time + 10 < ui_time)
	{
		ui_last_time = ui_time;

		unsigned char command = 0;

		if(cmd == 0) //Right
		{
			command = UI_4WAY_LEFT;
		}
		else if(cmd == 1) //Down
		{
			command = UI_4WAY_DOWN;
		}
		else if(cmd == 2) //Left
		{
			command = UI_4WAY_RIGHT;	
		}
		else if(cmd == 3) //Up
		{
			command = UI_4WAY_UP;		
		}
		else if(cmd == 4)
		{
			command = UI_KEY_OK;
		}
		else if(cmd == 5)
		{
			command = UI_KEY_BACK;
		}
		else if(cmd == 6)
		{
			command = UI_KEY_GP1;
		}
		else if(cmd == 7)
		{
			command = UI_KEY_GP2;
		}
		if(ui_routing[UI_ROUTE_XDI_COMMANDS_INTERNAL])
		{
			ui_cmd_callback(command);
		}
		if(ui_routing[UI_ROUTE_XDI_COMMANDS_EXTERNAL])
		{
			uiSubmitCommand(command);
		}
	}

}

void uiXdiPositionChange(unsigned int x, unsigned int y, unsigned char mask)
{
	if(ui_routing[UI_ROUTE_XDI_POSITION_INTERNAL])
	{
		ui_position_callback(uiGetRealX(x),uiGetRealY(y),mask);
	}
	if(ui_routing[UI_ROUTE_XDI_POSITION_EXTERNAL])
	{
		uiSubmitPosition(x,y,mask);
	}
}

void uiTouchPositionChange(unsigned int x, unsigned int y, unsigned char mask)
{
	if(ui_routing[UI_ROUTE_TOUCH_POSITION_INTERNAL])
	{
		ui_position_callback(uiGetRealX(x),uiGetRealY(y),mask);
	}
	if(ui_routing[UI_ROUTE_TOUCH_POSITION_EXTERNAL])
	{
		uiSubmitPosition(x,y,mask);
	}
}

void uiAccelerometerCommandChange(unsigned char command)
{
	if(ui_routing[UI_ROUTE_ACCELEROMETER_COMMANDS_INTERNAL])
	{
		ui_cmd_callback(command);
	}
	if(ui_routing[UI_ROUTE_ACCELEROMETER_COMMANDS_EXTERNAL])
	{
		uiSubmitCommand(command);
	}	
}

void uiCapifChange(capInterfaceKey state, char value)
{
	unsigned char command = 0;

	switch(state)
	{
		case CAPIF_KEY_OK:
			command = UI_KEY_OK;
			break;
		case CAPIF_KEY_BACK:
			command = UI_KEY_BACK;
			break;
		case CAPIF_KEY_GP1:
			command = UI_KEY_GP1;
			break;
		case CAPIF_KEY_GP2:
			command = UI_KEY_GP2;
			break;
		case CAPIF_WHEEL:
			if(value == 1)
			{
				command = UI_WHEEL_FORWARD;
			}
			else
			{
				command = UI_WHEEL_BACK;
			}
			break;			
	}
	
	if(ui_routing[UI_ROUTE_CAPACITIVE_COMMANDS_INTERNAL])
	{
		ui_cmd_callback(command);
	}
	if(ui_routing[UI_ROUTE_CAPACITIVE_COMMANDS_EXTERNAL])
	{
		uiSubmitCommand(command);
	}
}

void uiError(sieError error)
{
	
}

void uiSubmitPosition(unsigned int x, unsigned char y, unsigned char mask)
{
	unsigned char data[6];
	data[0] = x << 8; 	//MSB x
	data[1] = x & 0xFF;	//LSB x
	data[2] = y << 8;	//MSB y
	data[3] = y & 0xFF;	//LSB y
	data[4] = mask & 0x01?1:0;
	data[5] = mask & 0x02?1:0;

	uiPacketSubmit(0x51, 6, data);	 //Command 0x51 - absolute position
}

void uiSubmitCommand(unsigned char command)
{
	uiPacketSubmit(command, 0, 0);	
}

inline void uiReceive(unsigned char data)
{
	sieReceive(ui_sie_id, data);
}

inline void uiPacketSubmit(unsigned char command, unsigned char length, unsigned char *data)
{
	sieSubmit(ui_sie_id, command, length, data);
}

void uiSetCommandHandler(void (*cmd_callback)(unsigned char))
{
	ui_cmd_callback = cmd_callback?cmd_callback:uiSubmitCommand;
}

void uiSetPositionHandler(void (*position_callback)(unsigned int, unsigned int, unsigned char))
{
	ui_position_callback = position_callback!=0?position_callback:uiSubmitPosition;
}

void uiSetScreen(unsigned int width, unsigned int height)
{
	ui_screen_width = width;
	ui_screen_height = height;
}

void uiSetRouting(unsigned char key, unsigned char value)
{
	ui_routing[key]	= value;
}

void uiInit()
{
	sieInit(); 

	capInterfaceInit(uiCapifChange);
	xdiInit(uiXdiCommandChange, uiXdiPositionChange);
	touchInit(uiTouchPositionChange);
	accelerometerInit(uiAccelerometerCommandChange);

	ui_sie_id = sieCreate(uiPacketReceived, rs232Putchar0, uiError);

	rs232Init(0, BAUDRATE, uiReceive);
}

void uiMain()
{
	xdiMain();
	if(boardGetBTimer() > 30) //Asure that one cycle always takes 300ms
	{
		accelerometerMain();
		ui_time+=3; //Step up in 0.3s steps
		if(ui_time > 15000) //Reset!!!
		{
			ui_time = 0;
			ui_last_time = 0;
		}
		boardResetBTimer();
	}
}
