#ifndef _UI_H
	#define _UI_H 1

	#include "../sie/Sie.h"
	#include "CapInterface.h"

	typedef enum
	{
		UI_KEY_OK = 0x31,
		UI_KEY_BACK = 0x32,
		UI_KEY_GP1 = 0x33,
		UI_KEY_GP2 = 0x34,
		UI_WHEEL_FORWARD = 0x35,
		UI_WHEEL_BACK = 0x36,
		UI_4WAY_UP = 0x37,
		UI_4WAY_DOWN = 0x38,
		UI_4WAY_LEFT = 0x39,
		UI_4WAY_RIGHT = 0x3A,
	} uiCommands;

	enum
	{
		UI_ROUTE_TOUCH_POSITION_INTERNAL,
		UI_ROUTE_TOUCH_POSITION_EXTERNAL,
		UI_ROUTE_CAPACITIVE_COMMANDS_INTERNAL,
		UI_ROUTE_CAPACITIVE_COMMANDS_EXTERNAL,
		UI_ROUTE_XDI_COMMANDS_INTERNAL,
		UI_ROUTE_XDI_COMMANDS_EXTERNAL,
		UI_ROUTE_XDI_POSITION_INTERNAL,
		UI_ROUTE_XDI_POSITION_EXTERNAL,
		UI_ROUTE_ACCELEROMETER_COMMANDS_INTERNAL,
		UI_ROUTE_ACCELEROMETER_COMMANDS_EXTERNAL,	
	};


	/* 
	 * Initialize User Interface
	 */	
	void uiInit();

	/*
	 * Main Routine to be called periodically
	 */
	void uiMain();


	/*
	 * Sets a handler for all control commands
	 * send into the ui (CapIf, ...)
	 * @param cmd_callback	Function called, when a command is routed internally
	 */	
	void uiSetCommandHandler(void (*cmd_callback)(unsigned char));

	/*
	 * Sets a handler for all position changes
	 * send into the ui (Xdi, Touch, ...)
	 * @param cmd_callback	Function called, when position value is routed internally
	 */	
	void uiSetPositionHandler(void (*position_callback)(unsigned int, unsigned int, unsigned char));

	/*
	 * Set screen size to recalcualte the absolute Position
	 * @param width	New screen width
	 * @param height New screen height
	 */
	void uiSetScreen(unsigned int width, unsigned int height);

	/*
	 * Set screen size to recalcualte the absolute Position
	 * @param key	Key, identifying the routing event to be changed, see enum
	 * @param value	Value, 0 disable event, 1 enable event
	 */
	void uiSetRouting(unsigned char key, unsigned char value);

	/* Private function */

	//Callbacks
	void uiXdiCommandChange(unsigned char cmd);
	void uiXdiPositionChange(unsigned int x, unsigned int y, unsigned char mask);
	void uiTouchPositionChange(unsigned int x, unsigned int y, unsigned char mask);
	void uiAccelerometerCommandChange(unsigned char cmd);
	void uiCapifChange(capInterfaceKey state, char value);			

	//SIE wrapper functions
	void uiSubmitCommand(unsigned char command);
	void uiSubmitPosition(unsigned int x, unsigned char y, unsigned char mask);
	void uiPacketSubmit(unsigned char command, unsigned char length, unsigned char *data);
	void uiPacketReceived(siePacket packet);
	void uiReceive(unsigned char data);
	void uiError(sieError error);	


#endif
